#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2022-09-26
# File  : inv_cisco_cellular_lte
#
# inventory of Cisco cellular interfaces (SIM cards)
#
#

from typing import List
from dataclasses import dataclass
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    TableRow,
    OIDEnd,
    OIDBytes,
    all_of,
    contains,
    exists,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringByteTable,
    InventoryResult,
)


@dataclass
class CiscoCellularProfile:
    apn_type: str
    apn: str


_cisco_gsm_service_type = {
    '0': '1xRTT',
    '1': 'EVDO Revision 0',
    '2': 'EVDO Revision A',
    '3': 'EVDO Revision B',
    '4': 'GPRS',
    '5': 'EDGE',
    '6': 'UMTS/WCDMA',
    '7': 'HSDPA',
    '8': 'HSUPA',
    '9': 'HSPA',
    '10': 'HSPA Plus',
    '11': 'LTE TDD',
    '12': 'LTE FDD',
}


_cisco_gsm_band = {
    '1': 'unknown',
    '2': 'invalid',
    '3': 'none',
    '4': 'GSM-850',
    '5': 'GSM-900',
    '6': 'GSM-1800',
    '7': 'GSM-1900',
    '8': 'WCDMA-800',
    '9': 'WCDMA-850',
    '10': 'WCDMA-1900',
    '11': 'WCDMA-2100',
    '12': 'LTE Band',
}

_cisco_wan_cell_ext_protocol_type = {
    '1': 'unknown',
    '2': 'IPv4',
    '3': 'PPP',
    '4': 'IPv6',
    '5': 'IPv4v6',
}


def parse_inv_cisco_cellular_lte(string_table: List[StringByteTable]):
    section = []
    profiles = {}

    modem_table, profile_table = string_table

    for profile_oid_end, apn_type, apn in profile_table:
        radio_index, profile_index = profile_oid_end.split('.')
        profiles[radio_index] = CiscoCellularProfile(
            apn=apn,
            apn_type=apn_type
        )

    for oid_end, service_type, imsi, imei, iccid, msisdn, country, network, current_band in modem_table:
        entry = {
            'key_columns': {
                'iccid': iccid,
                'imei': imei,
            },
            'inventory_columns': {
                'imsi': imsi if imsi else None,
                'msisdn': msisdn if msisdn else None,
                'country': country if country else None,
                'network': network if network else None,
                'current_band': _cisco_gsm_band.get(current_band, f'Unknown {current_band}'),
                'apn': profiles[oid_end].apn if profiles.get(oid_end) else None,
                'apn_type': _cisco_wan_cell_ext_protocol_type.get(profiles[oid_end].apn_type) if profiles.get(oid_end) else None,
                'service_type': _cisco_gsm_service_type.get(str(service_type[0]*256 + service_type[1]))
            },
            'status_columns': {

            }
        }

        keys = list(entry['inventory_columns'].keys())
        for key in keys:
            if not entry['inventory_columns'][key]:
                entry['inventory_columns'].pop(key)

        section.append(entry)

    return section


def inventory_cisco_cellular_lte(params, section) -> InventoryResult:
    path = ['networking', 'cellular']

    for entry in section:
        yield TableRow(
            path=path,
            key_columns=entry['key_columns'],
            inventory_columns=entry['inventory_columns'],
            status_columns=entry['status_columns']
        )


register.snmp_section(
    name='inv_cisco_cellular_lte',
    parse_function=parse_inv_cisco_cellular_lte,
    fetch=[SNMPTree(
        base='.1.3.6.1.4.1.9.9.661.1',  # CISCO-WAN-3G-MIB::ciscoWan3gMIBObjects
        oids=[
            OIDEnd(),
            OIDBytes('1.1.5'),  # c3gCurrentServiceType
            '3.1.1.1',  # c3gImsi
            '3.1.1.2',  # c3gImei
            '3.1.1.3',  # c3gIccId
            '3.1.1.4',  # c3gMsisdn
            '3.2.1.8',  # c3gGsmCountry
            '3.2.1.9',  # c3gGsmNetwork
            '3.4.1.1.3',  # c3gGsmCurrentBand
        ]),
        SNMPTree(
            base='.1.3.6.1.4.1.9.9.817.1.1.2.3.1',  # CISCO-WAN-CELL-EXT-MIB::cwceLteProfileEntry
            oids=[
                OIDEnd(),
                '2',  # cwceLteProfileType
                '5',  # cwceLteProfileApn
            ])],
    detect=all_of(
        contains('.1.3.6.1.2.1.1.1.0', 'cisco'),  # sysDescr
        exists('.1.3.6.1.4.1.9.9.661.1.3.1.1.2.*'),  # CISCO-WAN-3G-MIB::c3gImei
    ))

register.inventory_plugin(
    name='inv_cisco_cellular_lte',
    inventory_function=inventory_cisco_cellular_lte,
    inventory_default_parameters={
    },
    inventory_ruleset_name='inv_cisco_cellular_lte',
)
